<?php

namespace NoccyLabs\ConLog;

use NoccyLabs\ConLog\Formatter\DefaultFormatter;
use NoccyLabs\ConLog\Sink\StreamSink;
use Psr\Log\LoggerInterface;
use Psr\Log\LoggerTrait;
use Stringable;

class ConsoleLogger implements LoggerInterface
{

    protected $sink;

    protected $formatter;

    use LoggerTrait;

    public function __construct(?callable $sink=null, ?callable $formatter=null)
    {
        $this->sink = ($sink ?? new StreamSink(STDOUT));
        $this->formatter = ($formatter ?? new DefaultFormatter());
    }

    public function log($level, string|Stringable $message, array $context = []): void
    {
        $formatted = call_user_func($this->formatter, $level, $message, $context) ?? $message;
        call_user_func($this->sink, $formatted."\n");
    }

    public function write(string $message): void
    {
        call_user_func($this->sink, $message);
    }

    public function writeln(string $message): void
    {
        call_user_func($this->sink, $message."\n");
    }
}

