<?php

namespace NoccyLabs\ConLog\Formatter;


class DefaultFormatter extends Formatter
{
    

    private string $timeFormat = "H:i:s";

    private string $lineFormat = "%time% <fg=gray>[</>%level%<fg=gray>]</> %message%";

    private ?int $levelLength = null;

    private bool $levelUppercase = false;

    private array $styles = [
        'time'              => [ "<fg=cyan>",               "</>" ],
        'level@notice'      => [ "<options=bold;fg=yellow>","</>" ],
        'level@warning'     => [ "<options=bold;fg=yellow>","</>" ],
        'level@error'       => [ "<options=bold;fg=red>",   "</>" ],
        'level@emergency'   => [ "<options=bold;fg=red>",   "</>" ],
        'message@notice'    => [ "<fg=yellow>",             "</>" ],
        'message@warning'   => [ "<fg=yellow>",             "</>" ],
        'message@error'     => [ "<fg=red>",                "</>" ],
        'message@emergency' => [ "<fg=red>",                "</>" ],
    ];

    public function timeFormat(string $timeFormat): self
    {
        $this->timeFormat = $timeFormat;
        return $this;
    }

    public function lineFormat(string $lineFormat): self
    {
        $this->lineFormat = $lineFormat;
        return $this;
    }

    public function levelLength(?int $length): self
    {
        $this->levelLength = $length;
        return $this;
    }

    public function levelUpper(bool $levelUppercase): self
    {
        $this->levelUppercase = $levelUppercase;
        return $this;
    }

    public function formatLine(string $level, string $message, array $context)
    {
        $output = preg_replace_callback('<%([a-z]+?)%>', function ($m) use ($level, $message, $context) {
            $text = match ($m[1]) {
                'time' => date($this->timeFormat),
                'level' => $this->levelLength ? str_pad(substr($level,0,$this->levelLength), $this->levelLength) : $level,
                'message' => strtr($message, array_combine(array_map(fn($k)=>'{'.$k.'}', array_keys($context)), array_values($context))),
                default => $context[$m[1]]??$m[0],
            };
            if ($m[1]=='level' && $this->levelUppercase) $text = strtoupper($text);
            return $this->formatElement($text, $m[1], $level);
        }, $this->lineFormat);

        return $output;
    }

    private function formatElement(string $text, string $element, string $level)
    {
        if (array_key_exists("{$element}@{$level}", $this->styles)) {
            $s = $this->styles["{$element}@{$level}"];
        } elseif (array_key_exists($element, $this->styles)) {
            $s = $this->styles["{$element}"];
        } elseif (array_key_exists("@{$level}", $this->styles)) {
            $s = $this->styles["@{$level}"];
        } else {
            return $text;
        }
        return sprintf("%s%s%s", $s[0], $text, $s[1]);
    }
}

