<?php

namespace NoccyLabs\ConLog\Sink;

class StreamSink
{
    protected $stream;

    protected bool $interactive;

    protected int $colors = self::COLORS_16;

    const COLORS_16 = 0;
    const COLORS_256 = 1;
    const COLORS_TRUE = 2;

    public function __construct($stream)
    {
        $this->stream = $stream;
        $this->interactive = posix_isatty($stream);
        if (str_contains(getenv("TERM"), "256")) {
            $this->colors = self::COLORS_256;
        }
    }

    public function colors(int $colordepth): self
    {
        $this->colors = ($colordepth>=0&&$colordepth<=2)?$colordepth:0;
        return $this;
    }

    public function __invoke(string $message)
    {
        if (!$this->interactive) {
            $message = preg_replace('[<(/|/{0,1}[a-z0-9=;,#]+?)>]', '', $message);
        } else {
            $message = preg_replace_callback('[<(/|/{0,1}[a-z0-9=;,#]+?)>]', function ($m) {
                //printf("[%s]\n", $m[1]);
                if ($m[1]=="/") {
                    return "\e[0m";
                }
                $reset = false;
                if (str_starts_with($m[1], "/")) {
                    $reset = true;
                    $m[1] = substr($m[1],1);
                }
                $attrs = explode(";", $m[1]);
                $sgr = [];
                foreach ($attrs as $attr) {
                    if (strpos($attr,"=")!==false) {
                        [$attr,$val] = explode("=",$attr, 2);
                    } else {
                        $val = null;
                    }
                    if ($reset) $attr = "/".$attr;
                    switch ($attr) {
                        case '/fg':
                            $sgr[] = 39;
                            break;
                        case '/bg':
                            $sgr[] = 49;
                            break;
                        case 'bg':
                            $bg = true;
                        case 'fg':
                            $bg ??= false;
                            array_push($sgr, ...$this->colorToSgr($val,$bg));
                            break;
                        case 'options':
                            $opts = explode(',', $val);
                            foreach ($opts as $opt) {
                                $r = match ($opt) {
                                    'bold' => 1,
                                    'dim'=> 2,
                                    'italic' => 3,
                                    'underscore' => 4,
                                    'blink' => 6,
                                    'reverse' => 7,
                                    'conceal' => 8,
                                    default => null
                                };
                                if ($r) $sgr[] = $r;
                            }
                            break;
                        case '/options':
                            $opts = explode(',', $val);
                            foreach ($opts as $opt) {
                                $r = match ($opt) {
                                    'bold' => 22,
                                    'dim'=> 22,
                                    'italic' => 23,
                                    'underscore' => 24,
                                    'blink' => 26,
                                    'reverse' => 27,
                                    'conceal' => 28,
                                    default => null
                                };
                                if ($r) $sgr[] = $r;
                            }
                            break;
                    }
                }
                return count($sgr) ? ("\e[".join(";",$sgr)."m") : "";
            }, $message);
        }
        fwrite($this->stream, $message);
    }

    private function colorToSgr(string $value, bool $isBackground)
    {
        if (str_starts_with($value, '#')) {
            if (strlen($value) == 7) {
                $cr = hexdec(substr($value, 1, 2));
                $cg = hexdec(substr($value, 3, 2));
                $cb = hexdec(substr($value, 5, 2));
            } elseif (strlen($value) == 4) {
                $cr = hexdec($value[1].$value[1]);
                $cg = hexdec($value[2].$value[2]);
                $cb = hexdec($value[3].$value[3]);
            } else {
                return [ 0 ];
            }
            if ($this->colors == self::COLORS_TRUE) {
                return [ $isBackground?48:38, 2, $cr, $cg, $cb ];
            } elseif ($this->colors == self::COLORS_256) {
                $cr = max(0,min(5,round($cr/51)));
                $cg = max(0,min(5,round($cg/51)));
                $cb = max(0,min(5,round($cb/51)));
                $cv = 36 * $cr + 6 * $cg + $cb;
                return [ $isBackground?48:38, 5, 16+$cv ];
            }
        } else {
            $cv = match ($value) {
                'black'         => 0,
                'red'           => 1,
                'green'         => 2,
                'yellow'        => 3,
                'blue'          => 4,
                'magenta'       => 5,
                'cyan'          => 6,
                'white'         => 7,
                'gray'          => 60,
                'bright-red'    => 61,
                'bright-green'  => 62,
                'bright-yellow' => 63,
                'bright-blue'   => 64,
                'bright-magenta'=> 65,
                'bright-cyan'   => 66,
                'bright-white'  => 67,
            };
            return [ $cv + ($isBackground?40:30) ];
        }
    }
}

